# Registers

!!! Attention
     resistor[ruh_**zi**_str] : [electrical component that implements electrical resistance](https://en.wikipedia.org/wiki/Resistor#:~:text=A%20resistor%20is%20a%20passive,transmission%20lines%2C%20among%20other%20uses.)    
     register[**reh**_juh_str] : [a memory location within the actual processor that work at very fast speeds](https://computersciencewiki.org/index.php/Registers_within_the_CPU)

## n-ary numbers

- 1 byte = 8 bits = [7:0]
- binary
    - 2進数(0,1)
    - 0b00001111
    - B00001111
- hexadecimal
    - 16進数(0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F)
    - 0xFF
    - $FF
- decimal
    - 10進数(0,1,2,3,4,5,6,7,8,9)
    - 0d255
    - 255
- ref. [2進数、8進数、10進数、16進数相互変換ツール](https://hogehoge.tk/tool/number.html)


!!! Note
    JP   | EN        |10       |255
    --   |---        |--       |--
    2進数 |Binary     |0b1010   |0b11111111  
    10進数|Decimal    |0d10　　　|0d255
    16進数|Hexadecimal|0xA      |0xff  

    1 byte = 8 bits = 2^8 = 256  
    1B = 8b  

    1**k**B = 1 [kilo] byte = 1000 B  
    1**K**B = 1 [ke̞ː] byte = 1024 B = 2^10 B = 1KiB = 1 [kibi] byte  

    1 bps = 1 [bits per second]  


## pinout
![](../images/register/ATtiny_x14.gif)

- Pin name
    - 5.1 Multiplexed Signals(page18)
        - Pin names are of type Pxn, with x being the PORT instance (A, B) and n the pin number. The notation for
signals is PORTx_PINn. All pins can be used as event input.  
    - PA0
        - PORT A
        - pin number 0

- Data Direction Register
    - 16.5.1 Data Direction
        - DDRA
            - PORT A
            - 0: input
            - 1: output

## Program

Arduio Sketch

```
void setup() {
  pinMode(8, OUTPUT);
}
void loop() {
  digitalWrite(8,HIGH);
  delay(500);
  digitalWrite(8,LOW);
  delay(500);
}
```

AVR
```
void setup() {
  DDRB |= 0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をOUTPUT
}
void loop() {
  PORTB |= 0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をHIGH
  delay(500);
  PORTB &= ~0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をLOW
  delay(500);
}
```

AVR 1-series
```
void setup() {
  PORTB.DIR |= 0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をOUTPUT
}
void loop() {
  PORTB.OUT |= 0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をHIGH
  delay(500);
  PORTB.OUT &= ~0b00000001;//ポートBの0番ピン(Arduinoの8番ピン)をLOW
  delay(500);
}
```

Ref. [Arduinoで入出力を高速化する方法](https://haizairenmei.com/2019/11/29/avr/)

!!! HINT
    // コメント  

    a = 0; //初期値 代入    
    b = a + 1; //代入   
    a = a + 1; //加算代入  
    a += a; //加算代入

    DDRB = 0b000000; //初期値  
    DDRB = DDRB | 0b00000001; //OR  
    DDRB |= 0b00000001; //OR  

OR  
```
0  0  1  1    operand1
0  1  0  1    operand2
----------
0  1  1  1    (operand1 | operand2) - returned result
```
1に指定したbitは、元が0でも1でも、1になる  -> 指定bitのセット  

NOT  
```
0  1    operand1
-----
1  0   ~operand1
```
1に指定したbitは、0に反転する (0は1に反転する)

AND  
```
0  0  1  1    operand1
0  1  0  1    operand2
----------
0  0  0  1    (operand1 & operand2) - returned result
```
NOTで反転した0に指定したbitは、元が0でも1でも、0になる  -> 指定bitのクリア  

ref. [Arduino Language Reference](https://www.arduino.cc/reference/en/)


Bitwise Operators (bit演算子)

|機能|演算子|使用例|
|-- |--    |--   |
|OR	|``|`` |指定bitのセット|
|AND|&	   |指定bitの取り出し, NOTと併用した指定bitのクリア|
|XOR|	^	|bit反転|
|NOT|	~	|bit反転, bitクリア|
|ビットシフト(右)|	>>	|ビットシフト|
|ビットシフト(左)|	<<	|ビットシフト|
